#include "epgsearchext.h"
#include "epgsearchcfg.h"
#include "epgsearchcats.h"
#include "tools.h"
#include <vdr/tools.h>
#include "menu_searchresults.h"
#include "menu_dirselect.h" 
#include "changrp.h"
#include "menu_search.h" 
#include "menu_searchedit.h" 
#include "menu_recsdone.h" 
#include "searchtimer_thread.h"
#include <math.h>
#ifdef HAVE_PCREPOSIX
#include <pcreposix.h>
#else
#include <regex.h>
#endif

cSearchExts SearchExts;
extern bool updateForced;

#define CURRENT_FILEVERS 2

// -- cSearchExt -----------------------------------------------------------------
char *cSearchExt::buffer = NULL;

cSearchExt::cSearchExt(void)
{
    ID = -1;
    *search = 0;
    options = 1;
    useTime = false;
    startTime = 0000;
    stopTime = 2359;
    useChannel = false;
    channelMin = Channels.GetByNumber(cDevice::CurrentChannel());
    channelMax = Channels.GetByNumber(cDevice::CurrentChannel());
    channelGroup = NULL; 
    useCase = false;
    mode = 0;
    useTitle = true;
    useSubtitle = true;
    useDescription = true;
    useDuration = false;
    useAsSearchTimer = false;
    useDayOfWeek = false;
    DayOfWeek = 0;
    buffer = NULL;
    *directory = 0;
    useEpisode = 0;
    Priority = EPGSearchConfig.DefPriority;
    Lifetime = EPGSearchConfig.DefLifetime;
    MarginStart = EPGSearchConfig.DefMarginStart;
    MarginStop = EPGSearchConfig.DefMarginStop;
    useVPS = false;
    announceOnly = false;
    useExtEPGInfo = false;
    catvalues = (char**) malloc(SearchExtCats.Count() * sizeof(char*));
    cSearchExtCat *SearchExtCat = SearchExtCats.First();
    int index = 0;
    while (SearchExtCat) 
    {
	catvalues[index] = (char*)malloc(MaxFileName);
	*catvalues[index] = 0;
	SearchExtCat = SearchExtCats.Next(SearchExtCat);
	index++;
    }
    avoidRepeats = 0;
    compareTitle = 1;
    compareSubtitle = 1;
    compareSummary = 1;
    allowedRepeats = 0;
    catvaluesAvoidRepeat = 0;
    repeatsWithinDays = 0;
    delAfterDays = 0;
}

cSearchExt::~cSearchExt(void)
{
  if (buffer) {
    free(buffer);
    buffer = NULL;
  }
}

cSearchExt& cSearchExt::operator= (const cSearchExt &SearchExt)
{
  memcpy(this, &SearchExt, sizeof(*this));
  return *this;
}

bool cSearchExt::operator< (const cListObject &ListObject)
{
  cSearchExt *SE = (cSearchExt *)&ListObject;
  return strcasecmp(search, SE->search) < 0;
}

const char *cSearchExt::ToText(void)
{
  char tmp_Start[5] = "";
  char tmp_Stop[5] = "";
  char tmp_minDuration[5] = "";
  char tmp_maxDuration[5] = "";
  char* tmp_chanSel = NULL;
  char* tmp_search = NULL;
  char* tmp_catvalues = NULL;

  free(buffer);
  if (mode < 4) // not a regular expression  
      asprintf(&tmp_search, "%s", search);
  else if (mode ==4)
  {
      tmp_search = strdup(search);
      while(strstr(tmp_search, "|"))
	  tmp_search = strreplace(tmp_search, "|", "!^pipe^!"); // ugly: replace with something, that should not happen to be part of a regular expression 
  }
  strreplace(tmp_search, ':', '|');
  strreplace(directory, ':', '|');

  if (useTime)
  {
    sprintf(tmp_Start, "%04d", startTime);
    sprintf(tmp_Stop, "%04d", stopTime);
  }
  if (useDuration)
  {
    sprintf(tmp_minDuration, "%04d", minDuration);
    sprintf(tmp_maxDuration, "%04d", maxDuration);
  }

  if (useChannel==1)
  {
      if (channelMin->Number() < channelMax->Number())
	  asprintf(&tmp_chanSel, "%s|%s", CHANNELSTRING(channelMin), CHANNELSTRING(channelMax));
      else
	  asprintf(&tmp_chanSel, "%s", CHANNELSTRING(channelMin));
  }
  if (useChannel==2)
  {
      int channelGroupNr = ChannelGroups.GetIndex(channelGroup);
      if (channelGroupNr == -1)
      {
	  esyslog("EPGSearch: channel group %s does not exist!", channelGroup);
	  useChannel = 0;
      }
      else
	  tmp_chanSel = strdup(channelGroup);
  }

  if (useExtEPGInfo)
  {
      cSearchExtCat *SearchExtCat = SearchExtCats.First();
      int index = 0;
      while (SearchExtCat) 
      {
	  char* catvalue = NULL;
	  asprintf(&catvalue, "%s", catvalues[index]);
	  while(strstr(catvalue, ":"))
	      catvalue = strreplace(catvalue, ":", "!^colon^!"); // ugly: replace with something, that should not happen to be part ofa category value
	  while(strstr(catvalue, "|"))
	      catvalue = strreplace(catvalue, "|", "!^pipe^!"); // ugly: replace with something, that should not happen to be part of a regular expression 

	  if (index == 0)
	      asprintf(&tmp_catvalues, "%d#%s", SearchExtCat->id, catvalue);
	  else
	  {
	      char* temp = tmp_catvalues;	      
	      asprintf(&tmp_catvalues, "%s|%d#%s", tmp_catvalues, SearchExtCat->id, catvalue);
	      free(temp);
	  }
	  SearchExtCat = SearchExtCats.Next(SearchExtCat);	 
	  index++;
	  free(catvalue);
      }      
  }

  asprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%ld:%d:%d\n",
	   ID,
	   tmp_search,
	   useTime,
	   tmp_Start,
	   tmp_Stop,
	   useChannel,
	   useChannel>0?tmp_chanSel:"0",
	   useCase,
	   mode,
	   useTitle,
	   useSubtitle,
	   useDescription,
	   useDuration,
	   tmp_minDuration,
	   tmp_maxDuration,
	   useAsSearchTimer,
	   useDayOfWeek,
	   DayOfWeek,
	   useEpisode,
	   directory,
	   Priority,
	   Lifetime,
	   MarginStart,
	   MarginStop,
	   useVPS,
	   announceOnly,
	   useExtEPGInfo,
	   useExtEPGInfo?tmp_catvalues:"",
	   avoidRepeats,
	   allowedRepeats,
	   compareTitle,
	   compareSubtitle,
	   compareSummary,
	   catvaluesAvoidRepeat,
	   repeatsWithinDays,
	   delAfterDays);

  strreplace(directory, '|', ':');

  if (tmp_chanSel)
      free(tmp_chanSel);
  if (tmp_search)
      free(tmp_search);
  if (tmp_catvalues)
      free(tmp_catvalues);


  return buffer;
}

bool cSearchExt::ParseV1(const char *s)
{
  char *line;
  char *pos;
  char *pos_next;
  int parameter = 1;
  int valuelen; 
  char value[MaxFileName];
  
  pos = line = strdup(s);
  pos_next = pos + strlen(pos);
  if (*pos_next == '\n') *pos_next = 0;
  while (*pos) {
    while (*pos == ' ') pos++;
    if (*pos) {
      if (*pos != ':') {
        pos_next = strchr(pos, ':');
        if (!pos_next)
          pos_next = pos + strlen(pos);
        valuelen = pos_next - pos + 1;
        if (valuelen > MaxFileName) valuelen = MaxFileName;
        strn0cpy(value, pos, valuelen);
        pos = pos_next;
        switch (parameter) {
	    case 1:  strcpy(search, value);
	      break;
	    case 2:  useTime = atoi(value);
	      break;
	    case 3:  startTime = atoi(value);
		break;
	    case 4:  stopTime = atoi(value);
	      break;
	    case 5:  useChannel = atoi(value);
		break;
	    case 6:  
		if (useChannel == 0)
		{
		    channelMin = NULL;
		    channelMax = NULL;
		}
		else if (useChannel == 1)
		{
		    int minNum=0, maxNum=0;
		    int fields = sscanf(value, "%d-%d", &minNum, &maxNum);
		    if (fields == 0) // stored with ID
		    {
			char *channelMinbuffer = NULL;
			char *channelMaxbuffer = NULL;
			int channels = sscanf(value, "%a[^|]|%a[^|]", &channelMinbuffer, &channelMaxbuffer);
			channelMin = Channels.GetByChannelID(tChannelID::FromString(channelMinbuffer), true, true);
			if (!channelMin) 
			{
			    esyslog("EPGSearch - ERROR: channel %s not defined", channelMinbuffer);
			    free(line);
			    return false;
			}
			if (channels == 1)
			    channelMax = channelMin;
			else
			{
			    channelMax = Channels.GetByChannelID(tChannelID::FromString(channelMaxbuffer), true, true);
			    if (!channelMax) 
			    {
				esyslog("EPGSearch - ERROR: channel %s not defined", channelMaxbuffer);
				free(line);
				return false;
			    }
			}

			free(channelMinbuffer);
			free(channelMaxbuffer);
		    }
		    else // stored with Number
		    {
			if (fields == 1)
			    maxNum = minNum;
			channelMin = Channels.GetByNumber(minNum);
			channelMax = Channels.GetByNumber(maxNum);
			if ((!channelMin && minNum>0) ||
			    (!channelMax && maxNum>0))
			{
			    free(line);
			    return false;
			}
		    }
		}
		else if (useChannel == 2)
		    channelGroup = strdup(value);
	      break;
	    case 7:  useCase = atoi(value);
	      break;
	    case 8:  mode = atoi(value);
		break;
	    case 9: useTitle = atoi(value);
		break;
	    case 10: useSubtitle = atoi(value);
		break;
	    case 11: useDescription = atoi(value);
		break;
	    case 12: useDuration = atoi(value);
		break;
	    case 13: minDuration = atoi(value);
		break;
	    case 14: maxDuration = atoi(value);
		break;
	    case 15: useAsSearchTimer = atoi(value);
			break;
	    case 16: useDayOfWeek = atoi(value);
			break;
	    case 17: DayOfWeek = atoi(value);
			break;
	    case 18: useEpisode = atoi(value);
			break;
	    case 19:  strcpy(directory, value);
	      break;
	    case 20: Priority = atoi(value);
			break;
	    case 21: Lifetime = atoi(value);
			break;
	    case 22: MarginStart = atoi(value);
			break;
	    case 23: MarginStop = atoi(value);
			break;
	    case 24: useVPS = atoi(value);
			break;
	    case 25: announceOnly = atoi(value);
			break;
	    case 26: useExtEPGInfo = atoi(value);
			break;
	    case 27: 
	    {
		if (!ParseExtEPGValues(value))
		{
		    esyslog("EPGSearch - ERROR reading ext. EPG values");
		    free(line);
		    return false;
		}
		break;
	    }
	    break;
        } //switch
      }
      parameter++;
    }
    if (*pos) pos++;
  } //while

  strreplace(directory, '|', ':');
  strreplace(search, '|', ':');
  while(strstr(search, "!^pipe^!"))
      strreplace(search, "!^pipe^!", "|");

  free(line);
  return (parameter >= 11) ? true : false;
}

bool cSearchExt::ParseV2(const char *s)
{
  char *line;
  char *pos;
  char *pos_next;
  int parameter = 1;
  int valuelen; 
  char value[MaxFileName];

  pos = line = strdup(s);
  pos_next = pos + strlen(pos);
  if (*pos_next == '\n') *pos_next = 0;
  while (*pos) {
    while (*pos == ' ') pos++;
    if (*pos) {
      if (*pos != ':') {
        pos_next = strchr(pos, ':');
        if (!pos_next)
          pos_next = pos + strlen(pos);
        valuelen = pos_next - pos + 1;
        if (valuelen > MaxFileName) valuelen = MaxFileName;
        strn0cpy(value, pos, valuelen);
        pos = pos_next;
        switch (parameter) {
	    case 1:  ID = atoi(value);
		break;
	    case 2:  strcpy(search, value);
	      break;
	    case 3:  useTime = atoi(value);
	      break;
	    case 4:  startTime = atoi(value);
		break;
	    case 5:  stopTime = atoi(value);
	      break;
	    case 6:  useChannel = atoi(value);
		break;
	    case 7:  
		if (useChannel == 0)
		{
		    channelMin = NULL;
		    channelMax = NULL;
		}
		else if (useChannel == 1)
		{
		    int minNum=0, maxNum=0;
		    int fields = sscanf(value, "%d-%d", &minNum, &maxNum);
		    if (fields == 0) // stored with ID
		    {
			char *channelMinbuffer = NULL;
			char *channelMaxbuffer = NULL;
			int channels = sscanf(value, "%a[^|]|%a[^|]", &channelMinbuffer, &channelMaxbuffer);
			channelMin = Channels.GetByChannelID(tChannelID::FromString(channelMinbuffer), true, true);
			if (!channelMin) 
			{
			    esyslog("EPGSearch - ERROR: channel %s not defined", channelMinbuffer);
			    free(line);
			    return false;
			}
			if (channels == 1)
			    channelMax = channelMin;
			else
			{
			    channelMax = Channels.GetByChannelID(tChannelID::FromString(channelMaxbuffer), true, true);
			    if (!channelMax) 
			    {
				esyslog("EPGSearch - ERROR: channel %s not defined", channelMaxbuffer);
				free(line);
				return false;
			    }
			}

			free(channelMinbuffer);
			free(channelMaxbuffer);
		    }
		}
		else if (useChannel == 2)
		    channelGroup = strdup(value);
	      break;
	    case 8:  useCase = atoi(value);
	      break;
	    case 9:  mode = atoi(value);
		break;
	    case 10: useTitle = atoi(value);
		break;
	    case 11: useSubtitle = atoi(value);
		break;
	    case 12: useDescription = atoi(value);
		break;
	    case 13: useDuration = atoi(value);
		break;
	    case 14: minDuration = atoi(value);
		break;
	    case 15: maxDuration = atoi(value);
		break;
	    case 16: useAsSearchTimer = atoi(value);
			break;
	    case 17: useDayOfWeek = atoi(value);
			break;
	    case 18: DayOfWeek = atoi(value);
			break;
	    case 19: useEpisode = atoi(value);
			break;
	    case 20:  strcpy(directory, value);
		break;
	    case 21: Priority = atoi(value);
			break;
	    case 22: Lifetime = atoi(value);
			break;
	    case 23: MarginStart = atoi(value);
			break;
	    case 24: MarginStop = atoi(value);
			break;
	    case 25: useVPS = atoi(value);
			break;
	    case 26: announceOnly = atoi(value);
			break;
	    case 27: useExtEPGInfo = atoi(value);
			break;
	    case 28: 
		if (!ParseExtEPGValues(value))
		{
		    esyslog("EPGSearch - ERROR reading ext. EPG values - 1");
		    free(line);
		    return false;
		}
		break;
	    case 29: avoidRepeats = atoi(value);
			break;
	    case 30: allowedRepeats = atoi(value);
			break;
	    case 31: compareTitle = atoi(value);
			break;
	    case 32: compareSubtitle = atoi(value);
			break;
	    case 33: compareSummary = atoi(value);
		break;
	    case 34: catvaluesAvoidRepeat = atol(value);
		break;
	    case 35: repeatsWithinDays = atoi(value);
		break;
	    case 36: delAfterDays = atoi(value);
		break;
        } //switch
      }
      parameter++;
    }
    if (*pos) pos++;
  } //while

  strreplace(directory, '|', ':');
  strreplace(search, '|', ':');
  while(strstr(search, "!^pipe^!"))
      strreplace(search, "!^pipe^!", "|");

  free(line);
  return (parameter >= 11) ? true : false;
}

char* cSearchExt::BuildFile(const cEvent* pEvent)
{
    char* file = NULL;

   if (!pEvent)
	return file;

    if (useEpisode)
    {
	char* pFile = NULL;
	if (!isempty(pEvent->ShortText()))
	    asprintf(&pFile, "%s~%s", pEvent->Title(), pEvent->ShortText());
	else
	    asprintf(&pFile, "%s~%s-%s", pEvent->Title(), GETDATESTRING(pEvent), GETTIMESTRING(pEvent));

	if (file) free(file);
	file = strdup(pFile);
	free(pFile);
    }
    else
	asprintf(&file, "%s", pEvent->Title());

    if (!isempty(directory))
    {
	char* pFile = NULL;

	if (strstri(directory, "%title%") == NULL &&
	    strstri(directory, "%subtitle%") == NULL)
	    asprintf(&pFile, "%s~%s", directory, file?file:"");
	else
	    // ignore existing title and subtitle in file if already used as variables in directory
	    asprintf(&pFile, "%s", directory);

	// replace ext. EPG variables
	cSearchExtCat* SearchExtCat = SearchExtCats.First();
	while (SearchExtCat) 
	{
	    char* tmp = NULL;
	    // search the category enclosed in %
	    asprintf(&tmp, "%%%s%%", SearchExtCat->name);
	    if (strstri(pFile, tmp))
	    {
		char* value = GetExtEPGValue(pEvent, SearchExtCat);
		if (value)
		    pFile = strreplacei(pFile, tmp, value);
	    }
	    free(tmp);
	    SearchExtCat = SearchExtCats.Next(SearchExtCat);
	}

	// replace Title, Subtitle, Channel vars
	pFile = strreplacei(pFile, "%title%", pEvent->Title()?pEvent->Title():"?");

	char* Subtitle = NULL;
	if (!isempty(pEvent->ShortText()))
	    asprintf(&Subtitle, "%s", pEvent->ShortText());
	else
	    asprintf(&Subtitle, "%s-%s", GETDATESTRING(pEvent), GETTIMESTRING(pEvent));
	pFile = strreplacei(pFile, "%subtitle%", Subtitle);
	free(Subtitle);
	
	cChannel *channel = Channels.GetByChannelID(pEvent->ChannelID(), true);
	if (channel)
	    pFile = strreplacei(pFile, "%channel%", CHANNELNAME(channel));

	if (file) free(file);
	file = strdup(pFile);
	free(pFile);
    }    
// replace some special chars
    if (file)
    {
	while(strstr(file, ":")) file = strreplace(file, ':', '|');
	while(strstr(file, " ~")) file = strreplace(file, " ~", "~");
	while(strstr(file, "~ ")) file = strreplace(file, "~ ", "~");
    }
    return file;
}

bool cSearchExt::ParseExtEPGValues(const char *s)
{
  char *line;
  char *pos;
  char *pos_next;
  int valuelen; 
  char value[MaxFileName];

  pos = line = strdup(s);
  pos_next = pos + strlen(pos);
  if (*pos_next == '\n') *pos_next = 0;
  while (*pos) {
    while (*pos == ' ') pos++;
    if (*pos) {
      if (*pos != '|') {
        pos_next = strchr(pos, '|');
        if (!pos_next)
          pos_next = pos + strlen(pos);
        valuelen = pos_next - pos + 1;
        if (valuelen > MaxFileName) valuelen = MaxFileName;
        strn0cpy(value, pos, valuelen);
        pos = pos_next;
	if (!ParseExtEPGEntry(value))
	{
	    esyslog("EPGSearch - ERROR reading ext. EPG value: %s", value);
	    free(line);
	    return false;
	}
      }
    }
    if (*pos) pos++;
  } //while

  free(line);
  return true;
}

bool cSearchExt::ParseExtEPGEntry(const char *s)
{
  char *line;
  char *pos;
  char *pos_next;
  int parameter = 1;
  int valuelen; 
  char value[MaxFileName];
  int currentid = -1;

  pos = line = strdup(s);
  pos_next = pos + strlen(pos);
  if (*pos_next == '\n') *pos_next = 0;
  while (*pos) {
    while (*pos == ' ') pos++;
    if (*pos) {
      if (*pos != '#') {
        pos_next = strchr(pos, '#');
        if (!pos_next)
          pos_next = pos + strlen(pos);
        valuelen = pos_next - pos + 1;
        if (valuelen > MaxFileName) valuelen = MaxFileName;
        strn0cpy(value, pos, valuelen);
        pos = pos_next;
        switch (parameter) {
	    case 1:  
		currentid = atoi(value);
	      break;
	    case 2:  
		if (currentid > -1)
		{
		    int index = SearchExtCats.GetIndexFromID(currentid);
		    if (index > -1)
		    {
			while(strstr(value, "!^colon^!"))
			    strreplace(value, "!^colon^!", ":");
			while(strstr(value, "!^pipe^!"))
			    strreplace(value, "!^pipe^!", "|");
			strcpy(catvalues[index], value);
		    }
		}
	      break;
        } //switch
      }
      parameter++;
    }
    if (*pos) pos++;
  } //while

  free(line);
  return (parameter >= 2) ? true : false;
}

bool cSearchExt::Save(FILE *f)
{
  return fprintf(f, "%s", ToText()) > 0;
}

cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEvent *Start, bool inspectTimerMargin) 
{
  cEvent *pe = NULL;
  cEvent *p1=schedules->Events()->First();
  // first we have to search for the Start item
  if(Start) {
     cEvent *p;
     for (p = schedules->Events()->First(); p && p!=Start; p = schedules->Events()->Next(p));
     if(p)
        p1=schedules->Events()->Next(p);
     }

  time_t tNow=time(NULL);
  char* szTest = NULL;
  char* searchText = strdup(search);

  int searchStart = 0, searchStop = 0;
  if (useTime)
  {
	searchStart = startTime;
	searchStop = stopTime;
	if (searchStop < searchStart)
		searchStop += 2400;
  }
  int minSearchDuration = 0;
  int maxSearchDuration = 0;
  if (useDuration)
  {
      minSearchDuration = minDuration/100*60 + minDuration%100;
      maxSearchDuration = maxDuration/100*60 + maxDuration%100;
  }

  for (cEvent *p = p1; p; p = schedules->Events()->Next(p))
  {
     if(!p) 
     {
        break;
     }

     if (szTest)
     {
	free(szTest);
	szTest = NULL;
     }

     // ignore events without title
     if (!p->Title() || strlen(p->Title()) == 0)
	 continue;

     asprintf(&szTest, "%s%s%s%s%s", (useTitle?p->Title():""), (useSubtitle||useDescription)?"~":"",
	      (useSubtitle?p->ShortText():""),useDescription?"~":"",
	      (useDescription?p->Description():""));     
	
     if (tNow < p->EndTime() + (inspectTimerMargin?(MarginStop * 60):0)) 
     {	    
        if (!useCase)
        {
	    ToLower(szTest);
	    ToLower(searchText);
        }
	
 	if (strlen(szTest) > 0)
	{
	    if (!MatchesSearchMode(szTest, searchText, mode," ,;|~"))
		continue;
	}
        if (useTime)
        {
		time_t tEvent = p->StartTime();
		struct tm tmEvent;
		localtime_r(&tEvent, &tmEvent);
		int eventStart = tmEvent.tm_hour*100 + tmEvent.tm_min;	
		int eventStart2 = tmEvent.tm_hour*100 + tmEvent.tm_min + 2400;	
		if ((eventStart < searchStart || eventStart > searchStop) &&
		    (eventStart2 < searchStart || eventStart2 > searchStop))
			continue;
        }
        if (useDuration)
        {
		int duration = p->Duration()/60;
		if (minSearchDuration > duration || maxSearchDuration < duration)
		    continue;
        }

	if (useDayOfWeek)
	{
	    time_t tEvent = p->StartTime();
	    struct tm tmEvent;
	    localtime_r(&tEvent, &tmEvent);
	    if (DayOfWeek >= 0 && DayOfWeek != tmEvent.tm_wday)
		continue;
	    if (DayOfWeek < 0)
	    {	      
		int iFound = 0;
		for(int i=0; i<7; i++)
		    if (abs(DayOfWeek) & (int)pow(2,i) && i == tmEvent.tm_wday)
		    {
			iFound = 1;
			break;
		    }
		if (!iFound)
		    continue;
	    }
	}
	if (useExtEPGInfo && !MatchesExtEPGInfo(p))
	    continue;
	pe=p;
	break;
     }
  }
  if (szTest)
	free(szTest);
  free(searchText);
  return pe;
}

// returns a pointer array to the matching search results
cSearchResults* cSearchExt::Run(bool noPayTV, bool inspectTimerMargin)
{
    LogFile.Log(3,"start search for search timer '%s'", search);

    cSchedulesLock schedulesLock;
    const cSchedules *schedules;
    schedules = cSchedules::Schedules(schedulesLock);
    if(!schedules) {
	LogFile.Log(1,"schedules are currently locked! try again later.");
	return NULL;
    }
    
    const cSchedule *Schedule = schedules->First();
    cSearchResults* pSearchResults = NULL;
    
    while (Schedule) {
	cChannel* channel = Channels.GetByChannelID(Schedule->ChannelID(),true);
	if (!channel)
	{
	    Schedule = (const cSchedule *)schedules->Next(Schedule);
	    continue;
	}
	
        if (useChannel == 1 && channelMin && channelMax)
	{
	    if (channelMin->Number() > channel->Number() || channelMax->Number() < channel->Number())
	    {
		Schedule = (const cSchedule *)schedules->Next(Schedule);
		continue;
	    }
	}
        if (useChannel == 2 && channelGroup)
	{
	    cChannelGroup* group = ChannelGroups.GetGroupByName(channelGroup);
	    if (!group || !group->ChannelInGroup(channel))
	    {
		Schedule = (const cSchedule *)schedules->Next(Schedule);
		continue;
	    }
	}
	
 	if (noPayTV) // no paytv
	{
	    if (channel->Ca() > CACONFBASE)
	    {
		Schedule = (const cSchedule *)schedules->Next(Schedule);
		continue;
	    }
	}
	
        const cEvent *pPrevEvent = NULL;
        do {
	    const cEvent* event = GetEventBySearchExt(Schedule, pPrevEvent,inspectTimerMargin);
	    pPrevEvent = event;
	    if (event && Channels.GetByChannelID(event->ChannelID()))
	    {
		if (!pSearchResults) pSearchResults = new cSearchResults;
		pSearchResults->Add(new cSearchResult(event));
	    }
        } while(pPrevEvent);
        Schedule = (const cSchedule *)schedules->Next(Schedule);
    }
    LogFile.Log(3,"found %d event(s) for search timer '%s'", pSearchResults?pSearchResults->Count():0, search);
	
    return pSearchResults;
}

void cSearchExt::CheckRepeatTimers(cSearchResults* pResults)
{
    if (!pResults)
	return;
    if (avoidRepeats == 0)
	return;

    LogFile.Log(2,"analysing repeats for search timer '%s'...", search);
    if (announceOnly)
	LogFile.Log(3,"search timer set to 'announce only', so skip all");

    cSearchResult* pResultObj = NULL;
    for (pResultObj = pResults->First(); pResultObj; pResultObj = pResults->Next(pResultObj))
    {
	const cEvent* pEvent = pResultObj->event;
	// check if this event was already recorded
	int records = 0;
	cRecDone* firstRec = NULL;
	records = RecsDone.GetCountRecordings(pEvent, this, &firstRec);

	if (announceOnly) // only announce if there is no timer for the event
	{
	    pResultObj->needsTimer = false; 		
	    continue;
	}
	
	if (records > allowedRepeats) // already recorded
	{
	    LogFile.Log(3,"skip '%s~%s' (%s - %s, channel %d): already recorded %d equal event(s)", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), records);
	    pResultObj->needsTimer = false; // first asume we need no timer
	    continue;
	}
	
	int plannedTimers = 0;
	cSearchResult* pFirstResultMatching = NULL;
	// check other results, if they are already planned for equal events
	for (cSearchResult* pResultObjP = pResults->First(); pResultObjP; pResultObjP = pResults->Next(pResultObjP))
	{
	    if (pResultObj == pResultObjP) break;
	    
	    const cEvent* pEventP = pResultObjP->event;
	    if (!pEventP) continue;
	    
	    if (!pResultObjP->needsTimer) continue;

	    if (EventsMatch(pEvent, pEventP, compareTitle, compareSubtitle, compareSummary, catvaluesAvoidRepeat))
	    {
		if (!pFirstResultMatching) pFirstResultMatching = pResultObjP;
		plannedTimers++;
	    }
	}
	
	if (plannedTimers + records > allowedRepeats)
	{
	    LogFile.Log(3,"skip '%s~%s' (%s - %s, channel %d): events planned(%d), recorded(%d), allowed(%d)", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), plannedTimers, records, allowedRepeats);
	    pResultObj->needsTimer = false; 		
	    continue;
	}
	else if (allowedRepeats > 0 && repeatsWithinDays > 0) // if we only allow repeats with in a given range
	{
	    if (firstRec) // already recorded, check for allowed repeat within days
	    {
		if (firstRec->startTime > pEvent->StartTime() - pEvent->Duration()) // no repeat
		{
		    LogFile.Log(3,"skip '%s~%s' (%s - %s, channel %d); no repeat for event already recorded at %s, channel %d", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), DAYDATETIME(firstRec->startTime), ChannelNrFromRecDone(firstRec));
		    pResultObj->needsTimer = false; 		
		    continue;
		}
		int daysFromFirstRec = int(double((pEvent->StartTime() - firstRec->startTime)) / (60*60*24) + 0.5);
		if (daysFromFirstRec  > repeatsWithinDays)
		{
		    LogFile.Log(3,"skip '%s~%s' (%s - %s, channel %d); first recording at %s is %d days before, limit is %d days", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), DAYDATETIME(firstRec->startTime),daysFromFirstRec, repeatsWithinDays);
		    pResultObj->needsTimer = false; 		
		    continue;
		}
	    }
	    if (plannedTimers > 0 && pFirstResultMatching)
	    {
		const cEvent* pFirst = pFirstResultMatching->event;
		if (pFirst->StartTime() > pEvent->StartTime() - pEvent->Duration()) // no repeat
		{
		    LogFile.Log(3,"skip '%s~%s' (%s - %s, channel %d); no repeat for event already recorded at %s - %s, channel %d", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), GETDATESTRING(pFirst), GETTIMESTRING(pFirst), ChannelNrFromEvent(pFirst));
		    pResultObj->needsTimer = false; 		
		    continue;
		}

		int daysBetween = int(double((pEvent->StartTime() - pFirst->StartTime())) / (60*60*24) + 0.5);
		if (daysBetween  > repeatsWithinDays)
		{
		    LogFile.Log(3,"skip '%s~%s' (%s - %s, channel %d); first event '%s~%s' (%s - %s) is %d days before, limit is %d days", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), GETDATESTRING(pFirst), GETTIMESTRING(pFirst),daysBetween, repeatsWithinDays);
		    pResultObj->needsTimer = false; 		
		    continue;
		}
	    }
	}
	bool dummy;
	cTimer* timer = cSearchTimerThread::GetTimer(this, pEvent, dummy);
	if (timer && !timer->HasFlags(tfActive))
	{
	    LogFile.Log(3,"skip '%s~%s' (%s - %s, channel %d), existing timer disabled", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
	    pResultObj->needsTimer = false; 		
	    continue;
	}
	else
	    LogFile.Log(3,"*** planning event '%s~%s' (%s - %s, channel %d) for recording", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
    }
    int needsTimer = 0;
    for (pResultObj = pResults->First(); pResultObj; pResultObj = pResults->Next(pResultObj))
	if (pResultObj->needsTimer) needsTimer++;

    LogFile.Log(2,"%d/%d events need a timer for search timer '%s'", needsTimer, pResults->Count(), search);
}

bool cSearchExt::MatchesExtEPGInfo(const cEvent* e)
{
    if (!e || !e->Description())
	return false;
    cSearchExtCat* SearchExtCat = SearchExtCats.First();
    while (SearchExtCat) 
    {
	char* value = NULL;
	int index = SearchExtCats.GetIndexFromID(SearchExtCat->id);
	if (index > -1)
	    value = catvalues[index];
	if (value && strlen(value) > 0)
	{
	    char* testvalue = GetExtEPGValue(e, SearchExtCat);
	    if (!testvalue)
		return false;

	    // compare not case sensitive
	    char* valueLower = strdup(value);
	    ToLower(valueLower);
	    ToLower(testvalue);
	    if (!MatchesSearchMode(testvalue, valueLower, SearchExtCat->searchmode, ",;|~"))
	    {
		free(testvalue);
		free(valueLower);
		return false;
	    }
	    free(testvalue);
	    free(valueLower);
	}
	SearchExtCat = SearchExtCats.Next(SearchExtCat);
    }
    return true;
}

// -- cSearchExts ----------------------------------------------------------------
bool cSearchExts::Load(const char *FileName)
{
    Clear();
    if (FileName) {
	free(fileName);
	fileName = strdup(FileName);
    }

    bool result = true;
    if (fileName && access(fileName, F_OK) == 0) {
	isyslog("loading %s", fileName);
	FILE *f = fopen(fileName, "r");
	if (f) {
	    int line = 0;
	    char buffer[MAXPARSEBUFFER];
	    result = true;
	    while (fgets(buffer, sizeof(buffer), f) > 0) {
                line++;
		char *p = strchr(buffer, '#');
		if (p)
		{
		    if (line == 1)
		    {
			if (strstr(p, "#version 2") == p)
			    fileversion = 2;
			*p = 0;
		    }
		}

                stripspace(buffer);
                if (!isempty(buffer)) {
		    cSearchExt* search = new cSearchExt;
		    if (fileversion == 1 && search->ParseV1(buffer))
			Add(search);
		    else if (fileversion == 2 && search->ParseV2(buffer))
			Add(search);
		    else {
			esyslog("ERROR: error in %s, line %d\n", fileName, line);
			delete search;
			result = false;
			break;
		    }
		}
	    }
	    fclose(f);
	}
	else {
	    LOG_ERROR_STR(fileName);
	    result = false;
	}
    }
    else
	fileversion = CURRENT_FILEVERS;
    if (!result)
	fprintf(stderr, "vdr: error while reading '%s'\n", fileName);
    else
    {
	if (fileversion < CURRENT_FILEVERS)
	{
	    isyslog("EPGSEARCH: file format %d will be changed to current version", fileversion);
	    if (!Backup())
		esyslog("EPGSEARCH: failed to create a backup of %s", fileName);
	    else
	    {
		fileversion = CURRENT_FILEVERS;
		Update();
		if (!Save())
		    esyslog("EPGSEARCH: failed to save new file format to %s", fileName);
		else
		    isyslog("EPGSEARCH: file format changed to version %d", fileversion);
	    }
	}
    }
    LogFile.Log(2,"loaded searches from %s (count: %d)", fileName, Count());
    return result;
}

int cSearchExts::GetNewID()
{
    int newID = -1;
    cSearchExt *l = (cSearchExt *)First();
    while (l) {
	newID = max(newID, l->ID);
	l = (cSearchExt *)l->Next();
    }
    return newID+1;
}

void cSearchExts::Update(void)
{
    cSearchExt *l = (cSearchExt *)First();
    while (l) {
	// check if ID is set
	if (l->ID == -1)
	    l->ID = GetNewID();
	l = (cSearchExt *)l->Next();
    }
}

bool cSearchExts::Save(void)
{
    bool result = true;
    cSearchExt *l = (cSearchExt *)this->First();
    cSafeFile f(fileName);
    if (f.Open()) {
	fprintf(f, "#version %d - DONT TOUCH THIS!\n", fileversion);	
	while (l) {
	    if (!l->Save(f)) {
		result = false;
		break;
	    }
	    l = (cSearchExt *)l->Next();
	}
	if (!f.Close())
	    result = false;
    }
    else
	result = false;
    return result;
}

bool cSearchExts::Backup(void)
{
    char* backup = NULL;
    asprintf(&backup, "%s.bak", fileName);
    bool bSuccess = CopyFile(fileName, backup);
    free(backup);
    return bSuccess;
}

cSearchExt* cSearchExts::GetSearchFromID(int ID)
{
    if (ID == -1)
	return NULL;
    cSearchExt *l = (cSearchExt *)First();
    while (l) {
	if (l->ID == ID)
	    return l;
	l = (cSearchExt *)l->Next();
    }
    return NULL;
}

