#ifndef _LOG_INC_
#define _LOG_INC_

#include <stdarg.h>
#include <stdio.h>
#include <vdr/tools.h>


class cLogFile: public cFile
{
    int loglevel;
 public:
    void cLogFile::Open(const char* filename, int LogLevel)
	{
	    loglevel = LogLevel;
	    if (loglevel == 0) return;
	    if (!cFile::Open(filename, O_CREAT|O_APPEND|O_WRONLY))
		esyslog("EPGSEARCH: could not open log file: %s", filename);
	    Log(1, "---------------------------------------", loglevel);
	    Log(1, "EPGSearch log started (verbose level %d)", loglevel);
	}
    void cLogFile::Log(int LogLevel, const char *text, ...)
	{
	    if (LogLevel > loglevel) return;
	    if(IsOpen()) 
	    {
		char* buffer = NULL;
		va_list Arg; 
		va_start(Arg,text); 
		vasprintf(&buffer, text, Arg); 
		va_end(Arg);
		time_t now = time(NULL);

		char datebuf[32];
		struct tm tm_r;
		tm *tm = localtime_r(&now, &tm_r);

		char *p = stpcpy(datebuf, WeekDayName(tm->tm_wday));
		*p++ = ' ';
		strftime(p, sizeof(datebuf) - (p - datebuf), "%d.%m.%Y", tm);
		
		char timebuf[25];
		strftime(timebuf, sizeof(timebuf), "%T", localtime_r(&now, &tm_r));

		char* log = NULL;
		asprintf(&log, "%s %s: %s\n", datebuf, timebuf, buffer);
		free(buffer);
		safe_write(*this, log, strlen(log));
		free(log);
	    }
	}
    int Level() { return loglevel; }
};

extern cLogFile LogFile;

#endif

 
