/*
 * menu_searchactions.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "menu_commands.h"
#include <vdr/interface.h>
#include <vdr/menu.h>
#include <vdr/plugin.h>
#include "menu_searchactions.h"
#include "menu_searchresults.h"
#include "menu_recsdone.h"
#include "tools.h"

extern int updateForced;
// --- cMenuSearchActions ---------------------------------------------------------

cMenuSearchActions::cMenuSearchActions(cSearchExt* Search, bool DirectCall)
:cOsdMenu(tr("Search actions"))
{
    directCall = DirectCall;
    SetHasHotkeys();
    
    search = Search;

    Add(new cOsdItem(hk(tr("Execute search"))));
    Add(new cOsdItem(hk(tr("Use as search timer on/off"))));
    Add(new cOsdItem(hk(tr("Trigger search timer update"))));
    Add(new cOsdItem(hk(tr("Show recordings done"))));
}

cMenuSearchActions::~cMenuSearchActions()
{
}


eOSState cMenuSearchActions::Search(void)
{
    return AddSubMenu(new cMenuSearchResults(search));
}

eOSState cMenuSearchActions::OnOffSearchtimer(void)
{
  if (search) 
  {
     search->useAsSearchTimer = search->useAsSearchTimer?0:1;
     SearchExts.Save();
  }
  return osBack;
}


eOSState cMenuSearchActions::Execute(void)
{
  int current = Current();
  if (current <= 3)
  {
     if (current == 0)
     	return Search();
     if (current == 1)
	return OnOffSearchtimer(); 
     if (current == 2)
     {
	 if (Interface->Confirm(tr("Run search timer update?")))
	     updateForced = 2; // with message about completion
	 return osBack;
     }	
     if (current == 3)
	 return AddSubMenu(new cMenuRecsDone(search));
  }
  return osContinue;
}

eOSState cMenuSearchActions::ProcessKey(eKeys Key)
{
    bool hadSubmenu = HasSubMenu();
    eOSState state = cOsdMenu::ProcessKey(Key);

    // jump back to calling menu, if a command was called directly with key '1' .. '9'
    if (directCall && hadSubmenu && !HasSubMenu())
	return osBack;

    if (state == osUnknown) {
	switch (Key) {

	    case kGreen:
	    case kYellow:
	    case kBlue:
		return osContinue;
	    case kOk:  
		return Execute();
	    default:   break;
	}
    }
    return state;
}
