/*
 * menu_searchresults.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */


#include "menu_searchresults.h"
#include "menu_myedittimer.h"
#include "menu_event.h"
#include "menu_commands.h"
#include "epgsearchcfg.h"
#include "tools.h"
#include "changrp.h"
#include "recdone.h"
#include "epgsearchcats.h"
#include <vdr/menu.h>

const char* ButtonBlue[3];

static int CompareRecording(const void *p1, const void *p2)
{
  return (int)((*(cRecording **)p1)->start - (*(cRecording **)p2)->start);
}

// --- cMenuSearchResultsItem -------------------------------------------------------
cMenuSearchResultsItem::cMenuSearchResultsItem(const cEvent *EventInfo, bool bEpisodeOnly, bool bPreviewTimer)
{
    fileName = NULL;
    eventInfo = EventInfo;
    char *buffer = NULL;
    cChannel *channel = Channels.GetByChannelID(eventInfo->ChannelID(), true);
    int TimerMatch;
    char t = Timers.GetMatch(EventInfo, &TimerMatch) ? (TimerMatch == tmFull) ? 'T' : 't' : ' ';
    if ((t == ' ' || t == 't') && bPreviewTimer)
	t = 'P';
    char v = eventInfo->Vps() && (eventInfo->Vps() - eventInfo->StartTime()) ? 'V' : ' ';
    char r = eventInfo->IsRunning() ? '*' : ' ';

    if(eventInfo->ShortText())
    {
	if (!bEpisodeOnly)
	    asprintf(&buffer, "%.*s\t%.*s\t%.*s\t%c%c%c\t%s%s%s", 10, CHANNELNAME(channel), 6, GETDATESTRING(eventInfo), 5, GETTIMESTRING(eventInfo), t, v, r, bEpisodeOnly?NULL:eventInfo->Title(), !isempty(eventInfo->ShortText())?" ~ ":"", !isempty(eventInfo->ShortText())?eventInfo->ShortText():"");
	else
	    asprintf(&buffer, "%.*s\t%.*s\t%.*s\t%c%c%c\t%s", 10, CHANNELNAME(channel), 6, GETDATESTRING(eventInfo), 5, GETTIMESTRING(eventInfo), t, v, r, eventInfo->ShortText());
    }
    else
	asprintf(&buffer, "%.*s\t%.*s\t%.*s\t%c%c%c\t%s", 10, CHANNELNAME(channel), 6, GETDATESTRING(eventInfo), 5, GETTIMESTRING(eventInfo), t, v, r, eventInfo->Title());

    if (EPGSearchConfig.showChannelNr)
    {
	char* tmpBuffer = NULL;
	asprintf(&tmpBuffer, "%d\t%s", channel->Number(), buffer);
	free(buffer);
	buffer = tmpBuffer;
    }
    SetText(buffer, false);
}

cMenuSearchResultsItem::cMenuSearchResultsItem(cRecording *Recording)
{
  fileName = strdup(Recording->FileName());
  SetText(Recording->Title('\t'));
}

// --- cMenuSearchResults -------------------------------------------------------

const cEvent *cMenuSearchResults::scheduleEventInfo = NULL;
int cMenuSearchResults::currentChannel = 0;

cMenuSearchResults::cMenuSearchResults(const char *p2, bool bSearchInRecordings)
:cOsdMenu(bSearchInRecordings ? tr("found recordings") : tr("additional starting times"), GetTab(1), GetTab(2), GetTab(3), GetTab(4), GetTab(5))
{
    ButtonBlue[0] = strdup(tr("all channels"));
    ButtonBlue[1] = strdup(tr("only FTA"));
    ButtonBlue[2] = strdup(tr("Timer preview"));

    // create a dummy search
    asprintf(&m_pSearchString, "%s", p2);
    searchExt = new cSearchExt;
    strcpy(searchExt->search, m_pSearchString);
    searchExt->mode = 0; // substring
    searchExt->useTitle = 1;
    searchExt->useSubtitle = 0;
    searchExt->useDescription = 0;

    m_bSort = true;
    m_bSearchInRecordings = bSearchInRecordings;
    modeYellow = showTitleEpisode;
    modeBlue = EPGSearchConfig.ignorePayTV?showNoPayTV:showAll;
    
    if (bSearchInRecordings)
	ScanRecordings();
    else
	BuildList();
}

cMenuSearchResults::~cMenuSearchResults()
{
    if (m_pSearchString)
    {
	delete searchExt;
	free(m_pSearchString);
    }
}

cMenuSearchResults::cMenuSearchResults(cSearchExt* SearchExt)
    :cOsdMenu(tr("Search results"), GetTab(1), GetTab(2), GetTab(3), GetTab(4), GetTab(5))
{
    ButtonBlue[0] = strdup(tr("all channels"));
    ButtonBlue[1] = strdup(tr("only FTA"));
    ButtonBlue[2] = strdup(tr("Timer preview"));

    searchExt = SearchExt;
    m_pSearchString = NULL;
    m_bSort = true;
    m_bSearchInRecordings = false;
    modeYellow = showTitleEpisode;
    modeBlue = EPGSearchConfig.ignorePayTV?showNoPayTV:showAll;
    
    BuildList();
}

int cMenuSearchResults::GetTab(int Tab)
{
    if (Tab==1)
	return EPGSearchConfig.showChannelNr?CHNUMWIDTH:12;
    if (Tab==2)
	return EPGSearchConfig.showChannelNr?12:7;
    if (Tab==3)
	return EPGSearchConfig.showChannelNr?7:6;
    if (Tab==4)
	return EPGSearchConfig.showChannelNr?6:3;
    if (Tab==5)
	return EPGSearchConfig.showChannelNr?3:30; 
    return 0;
}

bool cMenuSearchResults::BuildList(void)
{
  cSearchResults* pSearchResults = searchExt->Run(modeBlue == showNoPayTV);
  Clear();
  if (pSearchResults)
  {
      pSearchResults->SortBy(m_bSort? CompareEventTime: CompareEventChannel);

      if (modeBlue == showTimerPreview && searchExt->avoidRepeats)
	  searchExt->CheckRepeatTimers(pSearchResults);

      for (cSearchResult* pResultObj = pSearchResults->First(); 
	   pResultObj; 
	   pResultObj = pSearchResults->Next(pResultObj))
      {
	  if (!(searchExt->useAsSearchTimer && searchExt->avoidRepeats && modeBlue == showTimerPreview))
	      pResultObj->needsTimer = false;
	  Add(new cMenuSearchResultsItem(pResultObj->event, modeYellow == showEpisode, pResultObj->needsTimer));
      }

      delete pSearchResults;
  }
  SetHelpKeys();  
  char* szTitle = NULL;
  asprintf(&szTitle, "%d %s - %s", Count(), tr("Search results"), searchExt->search);
  SetTitle(szTitle);
  free(szTitle);
  Display();

  return true;
}

bool cMenuSearchResults::ScanRecordings(void)
{
    cRecording **pArray = NULL;
    int num = 0;
    
    Clear();
#if VDRVERSNUM <= 10310    
    cRecordings Recordings;
    bool recordings = Recordings.Load();
    if (!recordings)
	return true;
#endif

   for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording)) {
	if (strstr(recording->Title(), m_pSearchString) ||
	    strstr(m_pSearchString, recording->Title())) {
	    pArray = (cRecording **)realloc(pArray, (num + 1) * sizeof(cRecording *));
	    pArray[num++] = recording;
	}
    }

    qsort(pArray, num, sizeof(cRecording *), CompareRecording);

    for (int a = 0; a < num; a++)
	Add(new cMenuSearchResultsItem(pArray[a]));

    delete pArray;

    SetHelp(NULL);
    Display();
    return true;
}

void cMenuSearchResults::SetHelpKeys()
{
    ModeBlueSR nextModeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
    if (nextModeBlue == showTimerPreview && 
	(!searchExt || (searchExt && (searchExt->useAsSearchTimer == 0 || searchExt->avoidRepeats == 0))))
	nextModeBlue = (ModeBlueSR)(((int)nextModeBlue+1)%3);

    if (toggleKeys==0)
	SetHelp((EPGSearchConfig.redkeymode==0?tr("Record"):tr("Button$Commands")), m_bSort? tr("by channel"):tr("by time"), modeYellow==showTitleEpisode?tr("Episode"):tr("Title"), ButtonBlue[(int)nextModeBlue]);
    else
	SetHelp((EPGSearchConfig.redkeymode==1?tr("Record"):tr("Button$Commands")), m_bSort? tr("by channel"):tr("by time"), modeYellow==showTitleEpisode?tr("Episode"):tr("Title"), ButtonBlue[(int)nextModeBlue]);
}

cRecording *cMenuSearchResults::GetRecording(cMenuSearchResultsItem *Item)
{
#if VDRVERSNUM <= 10310    
    cRecordings Recordings;
    bool recordings = Recordings.Load();
    if (!recordings)
	return NULL;
#endif

  cRecording *recording = Recordings.GetByName(Item->FileName());
  if (!recording)
     Skins.Message(mtError, tr("Error while accessing recording!"));
  return recording;
}
eOSState cMenuSearchResults::Play(void)
{
  cMenuSearchResultsItem *ri = (cMenuSearchResultsItem*)Get(Current());
  if (ri) 
  {
      cRecording *recording = GetRecording(ri);
      if (recording) {
	  cReplayControl::SetRecording(recording->FileName(), recording->Title());
	  return osReplay;
      }
  }

  return osContinue;
}

eOSState cMenuSearchResults::Record(void)
{
  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->eventInfo);
     AddSubtitleToTimerFile(item->eventInfo, timer);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
#if VDRVERSNUM && VDRVERSNUM >= 10331
     if (EPGSearchConfig.useVDRTimerEditMenu)
	 return AddSubMenu(new cMenuEditTimer(timer, !t));
     else
#endif
     return AddSubMenu(new cMenuMyEditTimer(timer, !t, item->eventInfo));     
     }
  return osContinue;
}


eOSState cMenuSearchResults::Switch(void)
{
  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
  if (item) {
     cChannel *channel = Channels.GetByChannelID(item->eventInfo->ChannelID(), true);
     if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuSearchResults::Commands(eKeys Key)
{
  if (HasSubMenu() || Count() == 0)
     return osContinue;
  cMenuSearchResultsItem *mi = (cMenuSearchResultsItem *)Get(Current());
  if (mi && mi->eventInfo) {
        cMenuSearchCommands *menu;
        eOSState state = AddSubMenu(menu = new cMenuSearchCommands(tr("EPG Commands"), mi->eventInfo, true, searchExt));
        if (Key != kNone)
           state = menu->ProcessKey(Key);
        return state;
     }
  return osContinue;
}

eOSState cMenuSearchResults::ShowSummary()
{
    if (Count())
    {
	const cEvent *ei = ((cMenuSearchResultsItem *)Get(Current()))->eventInfo;
	if (ei) 
	{
	    cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
	    if (channel)
		return AddSubMenu(new cMenuEventSearch(ei, channel->Number() != cDevice::CurrentChannel()));
	}
    }
    return osContinue;
}

eOSState cMenuSearchResults::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
      if (!m_bSearchInRecordings)
      {
	  switch (Key) {
	      case kFastRew:
		  if(HasSubMenu()) 	     
		  {
		      CursorUp();
		      return ShowSummary();
		  }
		  break;
	      case kFastFwd:
		  if(HasSubMenu()) 	     
		  {
		      CursorDown();
		      return ShowSummary();
		  }
		  break;
	      case kRecord:
	      case kRed:
		  if(HasSubMenu()) {
		      state = Record();
		      break;
		  } 
		  if (Count())
		  {
		      if (EPGSearchConfig.redkeymode==toggleKeys)
			  state = Record();
		      else
		      {
			  cMenuSearchResultsItem* mi = (cMenuSearchResultsItem*)Get(Current());
			  if (mi) {
			      if (mi->eventInfo) {
				  return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->eventInfo, false, searchExt));
			      }
			  }
		      }
		  }
		  break;
	      case k0:
		  if(!HasSubMenu()) 
		  {
		      toggleKeys = 1 - toggleKeys;
		      SetHelpKeys();
		  }
		  state = osContinue;
		  break;
	      case k1...k9: 
		  if (!HasSubMenu())
		      return Commands(Key);
		  else
		      state = osContinue;
		  break;
	      case kGreen:  {
		  if(HasSubMenu()) 	     
		  {
		      CursorUp();
		      return ShowSummary();
		  }
		  else
		  {
		      m_bSort=!m_bSort;
		      BuildList();
		      state = osContinue;
		  }
	      }
		  break;
	      case kBlue:
		  if (HasSubMenu())
		      return Switch();
		  else
		  {
		      modeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
		      if (modeBlue == showTimerPreview && 
			  (!searchExt || (searchExt && (searchExt->useAsSearchTimer == 0 || searchExt->avoidRepeats == 0))))
			  modeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);

		      if (modeBlue == showTimerPreview)
			  m_bSort = true; // show always sorted by channel
		      BuildList();
		      state = osContinue;
		  }
		  break;
	      case kYellow: 
		  if(HasSubMenu()) 	     
		  {
		      CursorDown();
		      return ShowSummary();
		  }
		  else
		  {
		      modeYellow = (modeYellow==showTitleEpisode?showEpisode:showTitleEpisode);
		      BuildList();
		      state = osContinue;
		  }
		  break;
	      case kOk:
		  if(HasSubMenu()) 
		  {
		      state = cOsdMenu::ProcessKey(Key);
		      break;
		  }
		  if (Count())
		      return ShowSummary();
		  else
		      state = osBack;
		  break;
	      default:      
		  break;
	  }
      }
      else
      {
	  if (Key == kOk)
	      return Play();
	  else
	      state = osContinue;
      }
  }
  return state;
}


