/*
 * menu_whatson.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "menu_whatson.h"
#include "epgsearchext.h"
#include "menu_event.h"
#include "menu_myedittimer.h"
#include "menu_searchresults.h"
#include "menu_search.h" 
#include "menu_commands.h"
#include "epgsearchcfg.h"
#include "tools.h"
#include <math.h>
#include <vdr/menu.h>

#define HOURS(x) ((x)/100)
#define MINUTES(x) ((x)%100)

extern int exitToMainMenu;

// --- cMenuWhatsOnSearchItem ------------------------------------------------------
cMenuWhatsOnSearchItem::cMenuWhatsOnSearchItem(const cEvent *Event, cChannel *Channel, int iProgress, showMode ShowMode)
{
  event = Event;
  channel = Channel;
  char *buffer = NULL;
  if (!event)
  {
      if (iProgress > 0)
      {     
	  if (EPGSearchConfig.showChannelNr)
	      asprintf(&buffer, "%d\t%.*s\t%s\t\t\t%s%s%s", channel->Number(), 10, 
		       CHANNELNAME(channel), "", 
		       tr(">>> no info! <<<"), "", "");
	  else
	      asprintf(&buffer, "%.*s\t%s\t\t\t%s%s%s", 12, 
		       CHANNELNAME(channel), "", 
		       tr(">>> no info! <<<"), "", "");
      }
      else
      {
	  if (EPGSearchConfig.showChannelNr)
	      asprintf(&buffer, "%d\t%.*s\t%s\t\t%s%s%s", channel->Number(), 10, 
		       CHANNELNAME(channel), "", 
		       tr(">>> no info! <<<"), "", "");
	  else
	      asprintf(&buffer, "%.*s\t%s\t\t%s%s%s", 12, 
		       CHANNELNAME(channel), "", 
		       tr(">>> no info! <<<"), "", "");
      }
      SetText(buffer, false);
      return;
  }

  int TimerMatch;
  char t = Timers.GetMatch(Event, &TimerMatch) ? (TimerMatch == tmFull) ? 'T' : 't' : ' ';
  char v = event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
  char r = event->IsRunning() ? '*' : ' ';
  if (iProgress > 0)
  {
      int progress = 0;
      if (ShowMode == showNow)
	  progress = (int)roundf( (float)(time(NULL) - event->StartTime()) / (float)(event->Duration()) * 8.0 );
      if (ShowMode == showNext)
	  progress = (int)roundf( (float)(1 - min(float(30*60), float(event->StartTime() - time(NULL))) / (30*60)) * 8.0 );
      if (progress < 0) progress = 0;
      else if (progress > 8) progress = 8;

      char szProgress[9]="";
      memset(szProgress, 0, sizeof(szProgress));
      memset(szProgress, ' ', sizeof(szProgress)-1);
      for(int i = 0; i < progress; i++)
	  szProgress[i] = (iProgress == 1 ? '|' : 127);

      if (EPGSearchConfig.showChannelNr)
	  asprintf(&buffer, "%d\t%.*s\t%s\t%c%s%c\t %c%c%c \t%s%s%s", channel->Number(), 10, 
		   CHANNELNAME(channel), GETTIMESTRING(event), 
		   (iProgress==1?'[':128), szProgress, (iProgress==1?']':129), 
		   t, v, r, 
		   event->Title(), 
		   !isempty(event->ShortText())?" ~ ":"", 
		   !isempty(event->ShortText())?event->ShortText():"");
      else
	  asprintf(&buffer, "%.*s\t%s\t%c%s%c\t %c%c%c \t%s%s%s", 12, 
		   CHANNELNAME(channel), GETTIMESTRING(event), 
		   (iProgress==1?'[':128), szProgress, (iProgress==1?']':129), 
		   t, v, r, 
		   event->Title(),
		   !isempty(event->ShortText())?" ~ ":"", 
		   !isempty(event->ShortText())?event->ShortText():"");
  }
  else
      if (EPGSearchConfig.showChannelNr)
	  asprintf(&buffer, "%d\t%.*s\t%s\t %c%c%c \t%s%s%s", channel->Number(), 10, 
		   CHANNELNAME(channel), GETTIMESTRING(event), 
		   t, v, r, 
		   event->Title(), 
		   !isempty(event->ShortText())?" ~ ":"", 
		   !isempty(event->ShortText())?event->ShortText():"");
      else
	  asprintf(&buffer, "%.*s\t%s\t %c%c%c \t%s%s%s", 12, 
		   CHANNELNAME(channel), GETTIMESTRING(event), 
		   t, v, r, event->Title(), 
		   !isempty(event->ShortText())?" ~ ":"", 
		   !isempty(event->ShortText())?event->ShortText():"");
  SetText(buffer, false);
}


// --- cMenuWhatsOnSearch ----------------------------------------------------------

int cMenuWhatsOnSearch::currentChannel = 0;
showMode cMenuWhatsOnSearch::currentShowMode = showNow;
cChannel *cMenuWhatsOnSearch::scheduleChannel = NULL;
extern const char *ShowModes[];

cMenuWhatsOnSearch::cMenuWhatsOnSearch(const cSchedules *Schedules, int CurrentChannelNr)
    :cOsdMenu(EPGSearchConfig.ShowModes[currentShowMode].GetDescription(), GetTab(1), GetTab(2), GetTab(3), GetTab(4), GetTab(5))
{
    shiftTime = 0;
    schedules = Schedules;

    LoadSchedules();

    currentChannel = CurrentChannelNr;
    
    char* szTitle = NULL;
    asprintf(&szTitle, "%s - %s", tr("Overview"), EPGSearchConfig.ShowModes[currentShowMode].GetDescription());
    SetTitle(szTitle);
    free(szTitle);
    SetHelpKeys();
}

cMenuWhatsOnSearch::~cMenuWhatsOnSearch()
{
}

int cMenuWhatsOnSearch::GetTab(int Tab)
{
    if (Tab==1)
    {
	return EPGSearchConfig.showChannelNr?CHNUMWIDTH:6+CHNUMWIDTH;
    }
    if (Tab==2)
	return EPGSearchConfig.showChannelNr?10:6;
    if (Tab==3)
	if (currentShowMode==showNow || currentShowMode==showNext)
	    return EPGSearchConfig.showChannelNr?7:EPGSearchConfig.ProgressWidth;
	else
	    return EPGSearchConfig.showChannelNr?5:4;
    if (Tab==4)
	return EPGSearchConfig.showChannelNr?EPGSearchConfig.ProgressWidth:4;
    if (Tab==5)
	return 4;
    return 0;
}

void cMenuWhatsOnSearch::LoadSchedules()
{
    Clear();

    time_t SeekTime;
    char* szTitle = NULL;
    if (shiftTime != 0)
    {
	if (currentShowMode == showNow || currentShowMode == showNext)
	    SeekTime = time(NULL);
	else
	    SeekTime = GetTimeT(EPGSearchConfig.ShowModes[currentShowMode].GetTime());
	SeekTime += shiftTime*60;

	struct tm tm_r;
	time_t now = time(NULL);
	tm tm_seek = *localtime_r(&SeekTime, &tm_r);
	tm tm_now = *localtime_r(&now, &tm_r);
	if (tm_seek.tm_mday != tm_now.tm_mday)
	    asprintf(&szTitle, "%s - %s", tr("Overview"), DAYDATETIME(SeekTime));
	else
	    asprintf(&szTitle, "%s - %02d:%02d", tr("Overview"), tm_seek.tm_hour, tm_seek.tm_min);
    }
    else
	asprintf(&szTitle, "%s - %s", tr("Overview"), EPGSearchConfig.ShowModes[currentShowMode].GetDescription());
    
    SetTitle(szTitle);
    free(szTitle);
	
    for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) {
	if (!Channel->GroupSep()) {
	    const cSchedule *Schedule = schedules->GetSchedule(Channel->GetChannelID());
	    const cEvent *Event = NULL;
	    if (Schedule) 
	    {		
		if (shiftTime != 0)
		    Event = Schedule->GetEventAround(SeekTime);
		else
		{
		    switch(currentShowMode)
		    {  
			default:
			case showNow: 
			    Event = Schedule->GetPresentEvent();
			    break;
			case showNext:
			    Event = Schedule->GetFollowingEvent();
			    break;
			case showUserMode1:
			case showUserMode2:
			case showUserMode3:
			case showUserMode4:
			    time_t SeekTime = GetTimeT(EPGSearchConfig.ShowModes[currentShowMode].GetTime());
			    Event = Schedule->GetEventAround(SeekTime);
			    break;
		    }
		}
	     }
	     if (!EPGSearchConfig.showEmptyChannels && !Event)
		 continue;
	     int iProgress = 0;
	     if (currentShowMode==showNow)
		 iProgress = EPGSearchConfig.showProgress;
	     if (currentShowMode==showNext)
		 iProgress = EPGSearchConfig.showProgressNext;
	     Add(new cMenuWhatsOnSearchItem(Event, Channel, iProgress, currentShowMode), Channel->Number() == currentChannel);
	}
	else
	{
	    if (EPGSearchConfig.showChannelGroups)
	    {
		char* szGroup = NULL;
		asprintf(&szGroup, "----------------------------------------\t %s ----------------------------------------------------------------------------------------------", Channel->Name());
		cOsdItem* pGroupItem = new cOsdItem(szGroup);
		free(szGroup);
		pGroupItem->SetSelectable(false);
		Add(pGroupItem);
	    }
	}
    }
}

time_t cMenuWhatsOnSearch::GetTimeT(int iTime)
{
    struct tm tm_r;
    time_t t = time(NULL);
    tm* tmnow = localtime_r(&t, &tm_r);
    tmnow->tm_hour = HOURS(iTime);
    tmnow->tm_min = MINUTES(iTime);
    return mktime(tmnow);
}

showMode cMenuWhatsOnSearch::GetNextMode()
{
    showMode nextShowMode = currentShowMode;
    time_t now = time(NULL);
    do
    {
	nextShowMode = (showMode)((nextShowMode+1) % showModeMax);
	if (nextShowMode >= showUserMode1)
	{
	    if (!EPGSearchConfig.ShowModes[nextShowMode].GetUsage())
		continue;

	    time_t TestTime = GetTimeT(EPGSearchConfig.ShowModes[nextShowMode].GetTime());    
	    if (TestTime < now)
		continue;
	    break;
	}
	else
	    break;
    }
    while(true);
    return nextShowMode;
}
cChannel *cMenuWhatsOnSearch::ScheduleChannel(void)
{
  cChannel *ch = scheduleChannel;
  scheduleChannel = NULL;
  return ch;
}


eOSState cMenuWhatsOnSearch::Switch(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item && item->channel) {
     if (cDevice::PrimaryDevice()->SwitchChannel(item->channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuWhatsOnSearch::Record(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
      cTimer *timer = NULL;
      if (item->event)
      {
	  timer = new cTimer(item->event);
	  AddSubtitleToTimerFile(item->event, timer);
      }
      else
      {
	  time_t now = time(NULL); // thanks to nordlich@vdrportal for the following code, that
	  char *buffer=NULL;       // allows setting the channel for a timer without event
	  int starthh,startmm,stophh,stopmm;
	  //for getting the day of month
	  struct tm *tm_t=localtime(&now);
	  //we have to get away the ':' from the time string
	  sscanf(TIMESTRING(now),"%2d:%2d",&starthh,&startmm);
	  sscanf(TIMESTRING(now+Setup.InstantRecordTime*60),"%2d:%2d",&stophh,&stopmm);
	  //active:channel:day_of_month:starttime:endtime:priority:durability:title:
	  asprintf(&buffer,"1:%d:%d:%d%d:%d%d:%d:%d:%s:",item->channel->Number(),tm_t->tm_mday,starthh,startmm,stophh,stopmm,Setup.DefaultPriority,Setup.DefaultLifetime,item->channel->Name());
	  timer=new cTimer(false,false);
	  if(timer->Parse(buffer))
	      free(buffer);	      
      }
      cTimer *t = Timers.GetTimer(timer);
      if (t) {
	  delete timer;
	  timer = t;
      }
      timer->SetFlags(tfActive);
#if VDRVERSNUM && VDRVERSNUM >= 10331
      if (EPGSearchConfig.useVDRTimerEditMenu)
	  return AddSubMenu(new cMenuEditTimer(timer, !t));
      else
#endif
	  return AddSubMenu(new cMenuMyEditTimer(timer, !t, item->event, item->channel));
  }
  return osContinue;
}

eOSState cMenuWhatsOnSearch::Commands(eKeys Key)
{
    if (HasSubMenu() || Count() == 0)
	return osContinue;

    cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
    if (mi) 
    {
	// special case: switch to a channel without EPG
	if (!mi->event) 
	{
	    if (Key == k3)
		return Switch();
	    else if (Key == k2)
		return Record();
	    else
		return osContinue;
	}
        cMenuSearchCommands *menu;
        eOSState state = AddSubMenu(menu = new cMenuSearchCommands(tr("EPG Commands"), mi->event, true));
        if (Key != kNone)
	    state = menu->ProcessKey(Key);
        return state;
    }
    return osContinue;
}

eOSState cMenuWhatsOnSearch::ExtendedSearch(void)
{
    return AddSubMenu(new cMenuEPGSearchExt());
}

void cMenuWhatsOnSearch::SetHelpKeys()
{
    showMode nextShowMode = GetNextMode();    
    char* szButtonGreen = EPGSearchConfig.ShowModes[nextShowMode].GetDescription();
  if (toggleKeys==0)
      SetHelp((EPGSearchConfig.redkeymode==0?tr("Record"):tr("Button$Commands")), szButtonGreen, tr("Button$Schedule"), EPGSearchConfig.bluekeymode==0?tr("Switch"):tr("Button$Search"));
  else
      SetHelp((EPGSearchConfig.redkeymode==1?tr("Record"):tr("Button$Commands")), (EPGSearchConfig.toggleGreenYellow==0?szButtonGreen:"<<"), (EPGSearchConfig.toggleGreenYellow==0?tr("Button$Schedule"):">>"), EPGSearchConfig.bluekeymode==1?tr("Switch"):tr("Button$Search"));
}

eOSState cMenuWhatsOnSearch::Shift(int iMinutes)
{
    shiftTime += iMinutes;
    cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
    int TempChannel = currentChannel;
    if (mi) 
    {
	currentChannel = mi->channel->Number();
	scheduleChannel = Channels.GetByNumber(currentChannel);
    }
    LoadSchedules();
    Display();
    currentChannel = TempChannel;
    return osContinue;
}

eOSState cMenuWhatsOnSearch::ShowSummary()
{
    if (Count())
    {
	const cEvent *ei = ((cMenuWhatsOnSearchItem *)Get(Current()))->event;
	if (ei) 
	{
	    cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
	    if (channel)
		return AddSubMenu(new cMenuEventSearch(ei, channel->Number() != cDevice::CurrentChannel(), GetNextChannelWithEPG(-1), GetNextChannelWithEPG(1)));
	}
    }
    return osContinue;
}

const cChannel* cMenuWhatsOnSearch::GetNextChannelWithEPG(int direction)
{
    int currentOld = Current();
    int current = currentOld;
    do
    {	
	if (direction>0) current++; else current--;
	if (current<0 || current>=Count()) return NULL;
	cMenuWhatsOnSearchItem* item = (cMenuWhatsOnSearchItem *)Get(current);
	if (!item) return NULL;

	if (currentOld == current) // no move
	    return NULL;
	if (item->Selectable() && item->event)
	    return item->channel;
    }
    while(true);
}

void cMenuWhatsOnSearch::MoveToChannelWithEPG(int direction)
{
    
    const cChannel* ch = NULL;
    do
    {
	int currentOld = Current();
	if (direction>0)
	    CursorDown();
	else
	    CursorUp();
	if (currentOld == Current()) // no move
	    return;
	const cEvent *event = ((cMenuWhatsOnSearchItem *)Get(Current()))->event;	
	if (event)
	    return;
	else
	    ch = ((cMenuWhatsOnSearchItem *)Get(Current()))->channel;
    }
    while(ch);
}


eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
{
    exitToMainMenu = 0;
    if (!HasSubMenu() && Key == kBack)
    {
	exitToMainMenu = 1;
	return osBack;
    }

    eOSState state = cOsdMenu::ProcessKey(Key);

    if (state == osUnknown) {
	switch (Key) {
	    case kFastRew:
		if(HasSubMenu()) 	     
		{
		    MoveToChannelWithEPG(-1);
		    return ShowSummary();
		}
		else
		    return Shift(-EPGSearchConfig.timeShiftValue);
	    case kFastFwd:
		if(HasSubMenu()) 	     
		{
		    MoveToChannelWithEPG(1);
		    return ShowSummary();
		}
		else
		    return Shift(EPGSearchConfig.timeShiftValue);
	    case kRecord:
	    case kRed:
		if(HasSubMenu()) {
                        state = Record();
                        break;
                     } 
      		if (Count())
		{
			if (EPGSearchConfig.redkeymode==toggleKeys)
				state = Record();
			else
			{
				cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
	                        if (mi) {
        	                  if (mi->event) 
				     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
				  else
				      return osContinue;
				}
			}
		}
		break;

	 case k0:
	     if(!HasSubMenu()) 
	     {
		 toggleKeys = 1 - toggleKeys;
		 SetHelpKeys();
	     }
	     state = osContinue;
	     break;
	 case k1...k9: return Commands(Key);
	 case kYellow: 
	     if(HasSubMenu()) 	     
	     {
		 MoveToChannelWithEPG(1);
		 return ShowSummary();
	     }
	     else if (toggleKeys == 0 || (toggleKeys == 1 && EPGSearchConfig.toggleGreenYellow == 0))
	     {
		 cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
		 if (mi && mi->channel)
		 {
		     const cSchedule *Schedule = schedules->GetSchedule(mi->channel->GetChannelID());
		     if (Schedule)
		     {
			 time_t now = time(NULL);
			 bool hasFutureEvents = false;
			 for (const cEvent *e = Schedule->Events()->First(); e; e = Schedule->Events()->Next(e))
			     if (e->StartTime() > now)
			     {
				 esyslog("Start: %s", GETDATESTRING(e));
				 hasFutureEvents = true;
				 break;
			     }
			 if (!hasFutureEvents)
			     return osContinue;
		     }
		     else
			 return osContinue;
		 }
		 state = osBack;
		 // continue with kGreen
	     }
	     else
		 return Shift(EPGSearchConfig.timeShiftValue);
	 case kGreen:  
	 {
	     if(HasSubMenu()) 	     
	     {
		 MoveToChannelWithEPG(-1);
		 return ShowSummary();
	     }
	     else if (toggleKeys == 0 || (toggleKeys == 1 && EPGSearchConfig.toggleGreenYellow == 0))
	     {
		 if (Key == kYellow)
		     currentShowMode = showNow;
		 else
		     currentShowMode = GetNextMode();
		 cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
		 if (mi) 
		 {
		     currentChannel = mi->channel->Number();
		     scheduleChannel = Channels.GetByNumber(currentChannel);
		 }
	     }
	     else
		 return Shift(-EPGSearchConfig.timeShiftValue);
	 }
	 break;
	 case kBlue:   
	     if (HasSubMenu())
		 return Switch();
	     if (EPGSearchConfig.bluekeymode==toggleKeys)
		 return Switch();
	     else
		 return ExtendedSearch();
	   break;
	 case kOk:
	 {
	     cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
	     if (!mi->event) // no EPG, so simply switch to channel
		 return Switch();
	     else
		 return ShowSummary();
	 }
	 break;
       default:      break;
     }
  }
  return state;
}


