#include "tools.h"
#include "epgsearchext.h"
#include "epgsearchcats.h"
#include "epgsearchcfg.h"
#include "svdrpclient.h"
#include "distance.h"

#ifdef HAVE_PCREPOSIX
#include <pcreposix.h>
#else
#include <regex.h>
#endif
#include <ctype.h>

#define MATCHLIMIT 0.9

int CompareEventTime(const void *p1, const void *p2)
{
  return (int)((*(cSearchResult **)p1)->event->StartTime() - (*(cSearchResult **)p2)->event->StartTime());
}

int CompareEventChannel(const void *p1, const void *p2)
{
  return (int)( Channels.GetByChannelID((*(cSearchResult **)p1)->event->ChannelID())->Number() - Channels.GetByChannelID((*(cSearchResult **)p2)->event->ChannelID())->Number());
}

char* IndentMenuItem(const char* szString)
{
    char* szIndented = NULL;
    asprintf(&szIndented, "  %s", szString);
    return szIndented;
}

void PatchFont(eDvbFont DvbFont)
{
    const cFont *font = cFont::GetFont(DvbFont);
    int num_rows_global = font->Height();
    int num_rows = num_rows_global+2;
    int i,j,num_char;
    cFont::tPixelData* font_data = new cFont::tPixelData[225*(num_rows)];
    for (i = 0; i < 225; i++)
	for (j = 0; j < num_rows; j++)
	    font_data[(i*num_rows)+j]=0x0000000000000000;
    font_data[0+0]=6;//font->CharData(0)->width;
    font_data[0+1]=num_rows_global;
    for(num_char=33, i = 1; num_char<256; i++, num_char++)
    {
	const cFont::tCharData* char_data = font->CharData(num_char);
	font_data[i*num_rows+0] = (num_char>=127 && num_char<=129)?6:char_data->width;
	font_data[i*num_rows+1] = char_data->height;
	for(j = 0; j<(int)char_data->height; j++)
	{
	    cFont::tPixelData Data = (cFont::tPixelData)char_data->lines[j];
	    if (j>5 && j<(DvbFont==fontOsd?23:18))
	    {
		if (num_char == 127)
		    Data = (cFont::tPixelData)0x0000007f;
		if (num_char == 128)
		    Data = (cFont::tPixelData)0x0000000f;
		if (num_char == 129)
		    Data = (cFont::tPixelData)0x00000070;
	    }
	    font_data[(i*num_rows)+2+j] = Data;
	}
    }
    cFont::SetFont(DvbFont, font_data);
}

#if VDRVERSNUM <= 10310    
time_t LastModifiedTime(const char *FileName)
{
  struct stat fs;
  if (stat(FileName, &fs) == 0)
     return fs.st_mtime;
  return 0;
}
#endif

bool MatchesSearchMode(char* szTest, char* searchText, int mode, const char* delim)
{
    if (strlen(szTest) > 0)
    {
	if (mode == 0) // substring
	    return (strstr(szTest, searchText) != NULL); 
	if (mode == 1 || mode == 2) // AND or OR
	{ 
	    bool bTesting = false;
 	    char *pstrSearchToken, *pptr;
	    bool bFirst=true;
	    char *pstrSearch=strdup(searchText);
	    pstrSearchToken=strtok_r(pstrSearch, delim, &pptr);
	    while(pstrSearchToken) 
	    {
		if(szTest && strstr(szTest, pstrSearchToken)) 
		{ 
		    if(mode==1) 
		    { // means AND
			if(bFirst) 
			{
			    bTesting=true;
			    bFirst=false;
			} 
			else
			    bTesting&=true;
		    }
		    else
			bTesting|=true;
		} 
		else 
		{// not found!!
		    if(mode==1) 
		    { // means AND
			bTesting=false;
			bFirst=false;
		    }
		}
		pstrSearchToken=strtok_r(NULL, delim, &pptr);
	    }
	    free(pstrSearch);
	    return bTesting;
	}
	if (mode == 3) // match exactly
	{
	    if (strcmp(szTest, searchText) == 0)
		return true;
	    else
		return false;
	} 
	if (mode == 4)
	{
	    regex_t re;
	    bool bTesting = true;
	    if ( 0 == regcomp(&re, searchText, REG_EXTENDED) )
	    {
		if ( REG_NOMATCH == regexec( &re, szTest, 0, NULL, 0))
		{
		    regfree(&re);
		    return false;
		}
		else
		    bTesting = true;
		regfree(&re);
	    }
	    else
		bTesting = false;
	    return bTesting;
	}	
	return false;
    }
    return false;
}

void ToLower(char* szText)
{
    if (!szText)
	return;
    for (int loop = 0; szText[loop] !=0; loop++)
	szText[loop] = tolower(szText[loop]);
}

char* GetExtEPGValue(const cEvent* e, cSearchExtCat* SearchExtCat)
{
    if (!e || !SearchExtCat)
	return NULL;
    return GetExtEPGValue(e->Description(), SearchExtCat->name);
}

char* GetExtEPGValue(const char* description, const char* catname)
{
    if (isempty(description))
	return NULL;
    char* tmp = NULL;
    
    // search the category, must be at beginnig of a line
    asprintf(&tmp, "\n%s: ", catname);
    char* descr = strdup(description);
    char* cat = NULL;
    if ((cat = strstr(descr, tmp)) == NULL)
    {
	free(descr);
	free(tmp);
	return NULL;
    }
    else
	cat++; // skip linefeed

    // search the value to appear before the next line feed or end 
    char* end = strchr(cat, '\n');
    int endpos = strlen(cat);
    if (end)
	endpos = end - cat;
    cat[endpos] = 0;

    char* value = NULL;
    asprintf(&value, "%s", cat + strlen(tmp)-1);
    free(descr);
    free(tmp);

    return value;
}

// search s2 in s1 ignoring the case of s2
char* strstri(const char* s1, const char* s2)
{
    char* tmp_s1 = strdup(s1);
    char* tmp_s2 = strdup(s2);
    ToLower(tmp_s1);
    ToLower(tmp_s2);
    char* p = strstr(tmp_s1, tmp_s2);
    int of = -1;
    if (p)
	of = p - tmp_s1;
	
    free(tmp_s1);
    free(tmp_s2);
    
    if (of == -1)
	return NULL;
    else
	return (char*)(s1 + of);
}

// replace s1 with s2 in s ignoring the case of s1
char *strreplacei(char *s, const char *s1, const char *s2)
{
    char *p = strstri(s, s1);
    if (p) {
	int of = p - s;
	int l  = strlen(s);
	int l1 = strlen(s1);
	int l2 = strlen(s2);
	if (l2 > l1)
	    s = (char *)realloc(s, strlen(s) + l2 - l1 + 1);
	if (l2 != l1)
	    memmove(s + of + l2, s + of + l1, l - of - l1 + 1);
	strncpy(s + of, s2, l2);
    }
    return s;
}

void sleepMSec(long ms)
{
#if VDRVERSNUM >= 10314
    cCondWait::SleepMs(ms);
#else
    usleep(1000 * ms);
#endif
}

void sleepSec(long s)
{
    sleepMSec(s * 1000);
}

bool SendViaSVDRP(const char* SVDRPcmd) 
{
    bool bSuccess = true;
    char *cmdbuf = NULL;
    if (EPGSearchConfig.useExternalSVDRP)
    {
	asprintf(&cmdbuf, "%s -p %d \"%s\"", 
		 SVDRPSendCmd,
		 EPGSearchConfig.SVDRPPort,
		 SVDRPcmd);

	FILE *p = popen(cmdbuf, "r");
	if (p)
	    pclose(p);
	else
	{
	    esyslog("EPGSearch: can't open pipe for command '%s'", cmdbuf);
	    bSuccess = false;
	}
    }
    else
    {
	asprintf(&cmdbuf, "%s", SVDRPcmd);
	cSVDRPClient client(EPGSearchConfig.SVDRPPort);
	if (!client.SendCmd(cmdbuf))
	{
	    esyslog("EPGSearch: command '%s' failed", cmdbuf);
	    bSuccess = false;
	}
    }
	    
    free(cmdbuf);
    return bSuccess;
}

void SendMsg(const char* Message) 
{
    char *cmdbuf = NULL;
    asprintf(&cmdbuf, "MESG %s", Message);
    SendViaSVDRP(cmdbuf);
    free(cmdbuf);
}


bool InEditMode(const char* ItemText, const char* ItemName, const char* ItemValue)
{
    bool bEditMode = true;
    // ugly solution to detect, if in edit mode
    char* value = strdup(ItemText);
    strreplace(value, ItemName, "");
    strreplace(value, ":\t", "");
    // for bigpatch
    strreplace(value, "\t", "");
    if (strlen(value) == strlen(ItemValue))
	bEditMode = false;
    free(value);
    return bEditMode;
}

// checks if the timer was triggered from a search timer and return a pointer to the search
cSearchExt* TriggeredFromSearchTimer(cTimer* timer)
{
    if (!timer->Summary())
	return NULL;
    
    char* searchID = GetExtEPGValue(timer->Summary(), "S-ID");
    if (!searchID)
	return NULL;
    
    return SearchExts.GetSearchFromID(atoi(searchID));
}

bool CopyFile(char* file_path_from,  char* file_path_to)
{
    FILE* f_from;		/* stream of source file. */
    FILE* f_to;			/* stream of target file. */
    char buf[MAXPARSEBUFFER+1];   /* input buffer.          */

    /* open the source and the target files. */
    f_from = fopen(file_path_from, "r");
    if (!f_from) {
	esyslog("Cannot open source file: %s", file_path_from);
	return false;
    }
    f_to = fopen(file_path_to, "w+");
    if (!f_to) {
	esyslog("Cannot open target file: %s", file_path_to);
	return false;
    }

    /* copy source file to target file, line by line. */
    while (fgets(buf, MAXPARSEBUFFER+1, f_from)) {
	if (fputs(buf, f_to) == EOF) {  /* error writing data */
	    esyslog("Error writing to target file: %s", file_path_to);
	    return false;
	}
    }
    if (!feof(f_from)) { /* fgets failed _not_ due to encountering EOF */
	esyslog("Error reading from source file: %s", file_path_from);
	return false;
    }

    /* close source and target file streams. */
    if (fclose(f_from) == EOF) {
	esyslog("Error when closing source file: %s", file_path_from);
	return false;
	fprintf(stderr, "Error when closing source file: ");
	perror("");
    }
    if (fclose(f_to) == EOF) {
	esyslog("Error when closing target file: %s", file_path_to);
	return false;
    }
    return true;
}

double FuzzyMatch(const char* s1, const char* s2, int maxLength)
{
    Distance D;
    int dist = D.LD (s1, s2, maxLength);
    double fMaxLength = max(strlen(s1), strlen(s2));
    return (fMaxLength - dist)/fMaxLength;
}

bool DescriptionMatches(const char* eDescr, const char* rDescr)
{
    if (eDescr == NULL && rDescr == NULL) return true;
    if (eDescr == NULL && rDescr != NULL) return false;
    if (eDescr != NULL && rDescr == NULL) return false;
    if (strcmp(eDescr, rDescr) == 0) return true;

    // partial match:
    // first check the length, should only be different at MATCHLIMIT
    int minLength = min(strlen(eDescr), strlen(rDescr));
    int maxLength = max(strlen(eDescr), strlen(rDescr));
    if (double(minLength)/double(maxLength) < MATCHLIMIT)
	return false;
    // last try with Levenshtein Distance, only compare the first 1000 chars 
    double fMatch = FuzzyMatch(eDescr, rDescr, 1000);
    if (fMatch > MATCHLIMIT)
    {
	LogFile.Log(2,"match is: %.2f %%", fMatch*100);
	return true;
    }
    return false;
}

const cEvent* GetEvent(cTimer* timer)
{
    const cEvent* event = NULL;
    const cChannel *channel = timer->Channel();
    time_t Time = timer->StartTime() + (timer->StopTime() - timer->StartTime()) / 2;
    for (int seconds = 0; seconds <= 3; seconds++) 
    {
	{
	    cSchedulesLock SchedulesLock;
	    const cSchedules *Schedules = cSchedules::Schedules(SchedulesLock);
	    if (Schedules) {
		const cSchedule *Schedule = Schedules->GetSchedule(channel->GetChannelID());
		if (Schedule) {
		    event = Schedule->GetEventAround(Time);
		    if (event) return event; 
		}
	    }
	}
	if (seconds == 0)
	    LogFile.Log(2,"waiting for EPG info...");
	sleep(1);
    }
    LogFile.Log(1,"no EPG info available");
    return NULL;
}

// this extracts the real description from a given epg entry cutting all that looks like a category line
// we asume, that a category has a name not longer than MAXCATNAMELENGTH and a value not longer than 
// MAXCATVALUELENGTH (so in most cases e.g. the 'cast' category will stay part of the description). name and 
// value are separated with ': '
#define MAXCATNAMELENGTH 40
#define MAXCATVALUELENGTH 60
char* GetRawDescription(const char* descr)
{
    if (!descr) return NULL;

    char* rawDescr = NULL;
    const char* tmp = descr;    
    while(tmp)
    {
        // extract a single line
	char* line = strdup(tmp);
	char* lf = strchr(line, '\n');
	if (lf) *lf = 0;
	// check for category
	char* delim = strstr(line, ": ");
	if (!delim || 
	    (delim && (delim - line > MAXCATNAMELENGTH || strlen(line) - (delim - line) + 2 > MAXCATVALUELENGTH)))
	    if (strlen(line) > 0)
	    {
		if (rawDescr)
		{
		    char* tmp = strdup(rawDescr);
		    free(rawDescr); 
		    rawDescr = NULL;
		    asprintf(&rawDescr, "%s%s", tmp, line);
		    free(tmp);
		}
		else
		    rawDescr = strdup(line);
	    }
	
	if (lf) tmp += strlen(line)+1; else tmp = NULL;
	free(line);
    }
    return rawDescr;
}

void AddSubtitleToTimerFile(const cEvent* event, cTimer* timer)
{
    if (event && !isempty(event->ShortText())) // add subtitle if present
    {
	char* tmp = NULL;
	asprintf(&tmp, "%s~%s", event->Title(), event->ShortText());
	timer->SetFile(tmp);
	free(tmp);
    }
}

bool EventsMatch(const cEvent* event1, const cEvent* event2, bool compareTitle, bool compareSubtitle, bool compareSummary, unsigned long catvaluesAvoidRepeat)
{
    if (!event1 || !event2) return false;
    if (event1 == event2) return true;

    const char* Title1    = event1->Title();
    const char* Subtitle1 = event1->ShortText();
    const char* Descr1    = event1->Description();
    char* rawDescr1       = GetRawDescription(Descr1);
    
    const char* Title2    = event2->Title();
    const char* Subtitle2 = event2->ShortText();
    const char* Descr2    = event2->Description();
    char* rawDescr2       = GetRawDescription(Descr2);

    bool match = false;
    if ((!compareTitle || (!Title1 && !Title2) || (Title1 && Title2 && strcmp(Title2, Title1) == 0)) &&
	(!compareSubtitle || (!Subtitle1 && !Subtitle2) || (Subtitle1 && Subtitle2 && strcmp(Subtitle2, Subtitle1) == 0)) &&
	(!compareSummary || DescriptionMatches(rawDescr1, rawDescr2)))
    {	
	if (catvaluesAvoidRepeat != 0) // check categories
	{
	    bool bCatMatch = ((Descr1 && Descr2) || (!Descr1 && !Descr2));
	    cSearchExtCat *SearchExtCat = SearchExtCats.First();
	    int index = 0;
	    while (catvaluesAvoidRepeat > 0 && SearchExtCat && bCatMatch) 	    
	    {
		if (catvaluesAvoidRepeat & (1<<index))
		{
		    char* CatValue1 = GetExtEPGValue(Descr1, SearchExtCat->name);
		    char* CatValue2 = GetExtEPGValue(Descr2, SearchExtCat->name);
		    if ((!CatValue1 && CatValue2) ||
			(!CatValue2 && CatValue1) ||
			(CatValue1 && CatValue2 && strcmp(CatValue1, CatValue2) != 0))
			bCatMatch = false;
		    free(CatValue1);
		    free(CatValue2);
		}
		SearchExtCat = SearchExtCats.Next(SearchExtCat);
		index++;
	    }		    
	    if (bCatMatch)
		match = true;
	}
	else
	    match = true;
    }
    free(rawDescr1);
    free(rawDescr2);	    
    return match;
}

int ChannelNrFromEvent(const cEvent* pEvent)
{
    if (!pEvent)
	return -1;
    cChannel* channel = Channels.GetByChannelID(pEvent->ChannelID());
    if (!channel)
	return -1;
    else
	return channel->Number();						
}

int ChannelNrFromRecDone(cRecDone* recDone)
{
    if (!recDone)
	return -1;
    cChannel* channel = Channels.GetByChannelID(recDone->channelID);
    if (!channel)
	return -1;
    else
	return channel->Number();						
}

#if VDRVERSNUM <= 10324    
const char* TimeString(time_t t)
{
  static char buf[25] = "";
  struct tm tm_r;
  strftime(buf, sizeof(buf), "%R", localtime_r(&t, &tm_r));
  return buf;
}
#endif

#if VDRVERSNUM < 10318    
// --- cReadLine -------------------------------------------------------------
cReadLine::cReadLine(void)
{
  size = 0;
  buffer = NULL;
}

cReadLine::~cReadLine()
{
  free(buffer);
}

char *cReadLine::Read(FILE *f)
{
  int n = getline(&buffer, &size, f);
  if (n > 0) {
     n--;
     if (buffer[n] == '\n')
        buffer[n] = 0;
     return buffer;
     }
  return NULL;
}
#endif
